﻿' OakLib sample code for Visual Basic
'
' Copyright 2010 - Toradex AG
'
' The following code is provided as is, without any warranty


Imports System.Runtime.InteropServices


Module Oak

    ' 
    ' Step 1: translate the C data structure that are in the Oak.h header file (available in the OakLib package from the Toradex website).
    ' In order to work. The memory layout of the structures must match exactly the one of the C structures, hence the use MarshalAsAttribute attributes
    ' that help to cope with the specificities of C/C++ (string and array length for example).
    ' 

    '
    ' Declaration of the tChannel structure that is compatible with the C structure defined in  the C header file Oak.h
    ' 
    <StructLayoutAttribute(LayoutKind.Sequential, CharSet:=CharSet.Ansi)> _
    Public Structure tChannel
        <MarshalAsAttribute(UnmanagedType.I1)> Public IsSigned As Boolean
        Public BitSize As Byte
        Public UnitExponent As SByte
        Public Unit As UInteger
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public UnitStr As String
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public ChannelName As String
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public UserChannelName As String
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public UserChannelName_NV As String
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=64, ArraySubType:=UnmanagedType.I1)> Public RFU() As Byte
    End Structure

    '
    ' Declaration of the tOak sensor structure that is compatible with the C structure defined in the C header file Oak.h
    '
    <StructLayoutAttribute(LayoutKind.Sequential, CharSet:=CharSet.Ansi)> _
    Public Structure tOakSensor
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=256)> Public DevicePath As String
        Public VID As UShort
        Public PID As UShort
        Public REV As UShort
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public SN As String
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=64, ArraySubType:=UnmanagedType.U1)> Public RFU() As Byte
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public DeviceName As String
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public UserDeviceName As String
        <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=24)> Public UserDeviceName_NV As String
        Public NumChannels As UShort
        <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=11)> Public Channel() As tChannel
    End Structure


    Public Delegate Sub tNotifyFct(ByRef Sensor As tOakSensor, ByVal lParam As Integer) ' Declaration of the delegate function that will be used as the 'function pointer' for registereing callback

    '
    ' Step 2: Declare of the function that are present in the OakLib, With their parameter 'translated' to VB data types
    '
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Sub Oak_Initialize_a()
    End Sub
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Sub Oak_Hotplug_Enable_a()
    End Sub
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_Register_Callback_a(ByVal OnConnect As tNotifyFct, ByVal OnDisconnect As tNotifyFct, ByVal PID As UShort, ByVal REV As UShort, ByVal SN As String, ByVal DeviceName As String, ByVal ChannelName As String, ByVal UserDeviceName As String, ByVal UserChannelName As String, ByVal UserParam As Integer) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Sub Oak_DestroyNotifications_a()
    End Sub
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Sub Oak_NotifyConnectedDevices_a()
    End Sub
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_FindSensor_a(ByVal PID As UShort, ByVal REV As UShort, ByVal SN As String, ByVal DeviceName As String, ByVal ChannelName As String, ByVal UserDeviceName As String, ByVal UserChannelName As String, ByRef SensorFound As tOakSensor) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_Feature_a(ByVal DevicePath As String, ByVal RptBuffer() As Byte, ByVal ExpectResult As Boolean) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_GetInReport_a(ByVal DevicePath As String, ByVal RptBuffer() As Byte, ByVal InReportLength As Byte, ByVal Timeout_ms As UShort) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_GetCurrentInReport_a(ByVal DevicePath As String, ByVal RptBuffer() As Byte, ByVal InReportLength As Byte) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Function Oak_SendOutReport_a(ByVal DevicePath As String, ByVal RptBuffer() As Byte, ByVal InReportLength As Byte, ByVal Timeout_ms As UShort) As Boolean
    End Function
    <DllImport("oaka.dll", CallingConvention:=CallingConvention.Cdecl)> Public Sub Oak_Cleanup_a()
    End Sub

    '
    ' Convert Unsigned Byte to Signed Byte without generating overflow exception
    '
    Function ByteToSByte(ByVal b As Byte) As SByte
        Dim i As Integer = b
        If (i > 127) Then
            i = i - 256
        End If
        Return CSByte(i)
    End Function

    '
    ' Display information about the given sensor
    '
    Public Sub PrintSensorInfo(ByRef Sensor As tOakSensor)
        Console.WriteLine("===============================================")
        Console.WriteLine(Sensor.DeviceName & "(usr:" & Sensor.UserDeviceName & ")")
        Console.WriteLine("===============================================" & vbCrLf)
        Console.WriteLine("VID: " & Hex(Sensor.VID) & " PID: " & Hex(Sensor.PID) & " REV: " & Hex(Sensor.REV) & " SN: " & Sensor.SN & vbCrLf)
        Console.WriteLine("DevicePath: " & Sensor.DevicePath & vbCrLf)
        Console.WriteLine(Sensor.NumChannels & " data channels")

        Dim RptIdx As Integer = 1
        Dim i As Integer
        For i = 0 To Sensor.NumChannels - 1
            Console.WriteLine(vbCrLf & "  ----------")
            Console.WriteLine("  Channel " & i)
            Console.WriteLine("  ----------")
            Console.WriteLine("  " & Sensor.Channel(i).ChannelName & " (usr:" & Sensor.Channel(i).UserChannelName & ")")
            Console.WriteLine("  " & Sensor.Channel(i).BitSize & " bit" & CStr(IIf(Sensor.Channel(i).IsSigned, " signed", " unsigned")))
            Console.WriteLine("  Resolution: 1e" & Sensor.Channel(i).UnitExponent & " " & Sensor.Channel(i).UnitStr)
        Next
    End Sub

    '
    ' Print the current state of the given sensor
    '
    Public Sub PrintSensorValues(ByRef Sensor As tOakSensor)
        ' First we try obtain obtain the current status of the sensor by reading an In Report
        Dim Report(33) As Byte
        If (Not Oak_GetInReport_a(Sensor.DevicePath, Report, 33, 10000)) Then
            Console.WriteLine("Error: Oak_GetInReport_a() failed!")
            End
        End If

        ' The we use the information given in the Channel list of the sensor structure to parse the report into the actual values
        Console.WriteLine(vbCrLf & "  Current sensor values are:")
        Console.WriteLine("  --------------------------")
        Dim RptIdx = 1 ' byte 0 of the report is the report number and is always 0. 
        Dim i As Integer
        Dim Val As Double
        For i = 0 To Sensor.NumChannels - 1 ' For each channel
            Select Case Sensor.Channel(i).BitSize ' depending on it bit size and signed-ness, rebuild the correct value
                Case 8
                    If Sensor.Channel(i).IsSigned Then
                        Val = CDbl(ByteToSByte(Report(RptIdx)))
                    Else
                        Val = CDbl(Report(RptIdx))
                    End If
                    RptIdx += 1
                Case 16
                    If Sensor.Channel(i).IsSigned Then
                        Val = CDbl(CInt(ByteToSByte(Report(RptIdx))) + 256 * CInt(ByteToSByte(Report(RptIdx + 1))))
                    Else
                        Val = CDbl(CInt(Report(RptIdx)) + 256 * CInt(Report(RptIdx + 1)))
                    End If
                    RptIdx += 2
            End Select
            Val *= 10 ^ (Sensor.Channel(i).UnitExponent) ' apply the channel unit exponent
            Console.WriteLine("  " & Sensor.Channel(i).UserChannelName & ": " & Val & Sensor.Channel(i).UnitStr)
        Next
    End Sub


    '
    ' Set the device reporting mode to 'After Sample' and the sampling rate to the given value
    ' For more details of the signification of the reporting rate and sampling mode, please refer
    ' to the Oak Sensors programming guide and your device's datasheet
    Public Sub SetSamplingParams(ByRef Sensor As tOakSensor, ByVal rate As UShort)
        ' First, set the report mode
        ' the format of the feature report is described in the devices' datasheets
        Dim report(33) As Byte
        report(0) = 0 ' Report #, always 0
        report(1) = 0 ' Set
        report(2) = 0 ' To RAM
        report(3) = 1
        report(4) = 0
        report(5) = 0
        report(6) = 0 ' "After Sampling" mode
        If Not Oak_Feature_a(Sensor.DevicePath, report, False) Then
            Console.WriteLine("Error: Oak_Feature_a() failed!")
            End
        End If

        ' Then Set the sample rate
        report(0) = 0 ' Report #, always 0
        report(1) = 0 ' Set
        report(2) = 0 ' To RAM
        report(3) = 2
        report(4) = 1
        report(5) = 0
        report(6) = CByte(rate Mod 256) ' LSB of the rate
        report(7) = CByte(rate >> 8) ' MSB of the rate
        If Not Oak_Feature_a(Sensor.DevicePath, report, False) Then
            Console.WriteLine("Error: Oak_Feature_a() failed!")
            End
        End If

    End Sub


    '
    ' Connect callback. This functions will be called every time an Oak sensor is connected to the system
    '
    Public Sub ConnectCallback(ByRef Sensor As tOakSensor, ByVal lParam As Integer)
        Console.WriteLine(vbCrLf & vbCrLf & "--> Connected: <--")
        SetSamplingParams(Sensor, 50)
        PrintSensorInfo(Sensor)
        PrintSensorValues(Sensor)
    End Sub

    '
    ' Disconnect callback. This functions will be called every time an Oak sensor is disconnected from the system
    '
    Public Sub DisconnectCallback(ByRef Sensor As tOakSensor, ByVal lParam As Integer)
        Console.WriteLine(vbCrLf & vbCrLf & "--> Disconnected: <--\n")
        PrintSensorInfo(Sensor)
    End Sub

    '
    ' Main function
    '
    Sub Main()
        Oak_Initialize_a() ' Initialize the OakLib library
        Oak_Hotplug_Enable_a() ' Enable the notifications for the connecting/disconnecting of Oak devices
        Oak_Register_Callback_a(AddressOf ConnectCallback, AddressOf DisconnectCallback, 0, 0, "", "", "", "", "", 0) ' Register 2 callback to be called when connect/disconnect event occurs
        Oak_NotifyConnectedDevices_a() ' Ask the already connected devices to generate a connection event
        Console.ReadKey() ' wait until the user press a key. During this waiting period, Calla
        Oak_Cleanup_a() ' cleanup
    End Sub


End Module
